;;; texfax.el -- yet another `fax-write-region-function'.
;;;
;;; Copyright (C) 1995 Ralph Schleicher
;;;
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License as
;;; published by the Free Software Foundation; either version 2 of
;;; the License, or (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;;;
;;; This file is not part of GNU Emacs.
;;;
;;; Author: Ralph Schleicher <rs@purple.IN-Ulm.DE>
;;; Maintainer: see the `Author' field
;;; Keywords: local comm fax
;;; Comments: You can setup this code with, for example,
;;;
;;;	(require 'texfax)
;;;
;;;	(setq sendfax-write-region-function 'texfax-write-region)
;;;
;;;	(setq fax-mode-hook
;;;	      '(lambda ()
;;;		 (setq fill-column 76))
;;;
;;;	      fax-setup-hook
;;;	      '(lambda ()
;;;		 (save-excursion
;;;		   (let ((start (point)))
;;;		     (insert "\\epsffile{/full/path/to/signature.ps}\n")
;;;		     (put-text-property start (point) 'intangible t))))
;;;
;;;	(setq texfax-dvips-switches '("-P" "fax"))
;;;
;;; Time-stamp: "Thu Dec 14 18:13:45 MET 1995 rs@purple.IN-Ulm.DE"
;;; Code:


(defvar texfax-preamble "\
\\input epsf

\\catcode`\\@=11

% Font selection.
%
% dctt12: body_height=8.33, cap_height=7.33, desc_depth=2.67
%
\\font\\monospaced dctt12
\\textfont\\z@\\monospaced
\\fam\\z@\\monospaced

\\setbox\\strutbox\\hbox{\\vrule height 9\\p@ depth 4\\p@ width\\z@}
\\normalbaselineskip 13\\p@ \\normalbaselines

% Page layout.
%
\\hoffset -1in \\advance\\hoffset 60pt
\\voffset -1in \\advance\\voffset 30pt

\\hsize 40pc
\\vsize 54\\normalbaselineskip

\\overfullrule\\z@
\\frenchspacing
\\nopagenumbers
\\raggedbottom

\\clubpenalty 4000
\\widowpenalty\\@M

% Macros.
%
\\parindent\\z@
\\parskip\\z@

\\def\\verbatim{\\begingroup
  \\everypar{\\setbox\\z@\\hbox\\bgroup\\strut}%
  \\def\\par{\\leavevmode\\egroup\\box\\z@\\endgraf}%
  \\def\\do##1{\\catcode`##1=12}\\dospecials
    \\catcode`\\\\=0 \\chardef\\\\=`\\\\
  \\obeylines\\obeyspaces\\verb@tim}
{\\obeylines\\gdef\\verb@tim#1^^M{}}
\\def\\endverbatim{\\endgroup\\ignorespaces}
{\\obeyspaces\\gdef {\\ }}

% Hide private definitions.
%
\\catcode`\\@=12

% Latin-1 character encoding.
%
\\catcode`\\^^ab=\\active\\def ^^ab{\\char`\\^^13}	% \\flqq
\\catcode`\\^^bb=\\active\\def ^^bb{\\char`\\^^14}	% \\frqq
\\catcode`\\^^a7=\\active\\def ^^a7{\\char`\\^^9f}	% \\S
\\catcode`\\^^ff=\\active\\def ^^ff{\\char`\\^^b8}	% \\\"y
\\catcode`\\^^a1=\\active\\def ^^a1{\\char`\\^^bd}	% !`
\\catcode`\\^^bf=\\active\\def ^^bf{\\char`\\^^be}	% ?`
\\catcode`\\^^a3=\\active\\def ^^a3{\\char`\\^^bf}	% \\pounds
\\catcode`\\^^df=\\active\\def ^^df{\\char`\\^^ff}	% \\ss\n"
  "*Text inserted at the beginning of the TeX buffer.
`texfax-preamble' should end in a newline.")

(defvar texfax-postamble "\\bye\n"
  "*Text inserted at the end of the TeX buffer.
`texfax-postamble' should end in a newline.")

(defvar texfax-prefix "\\verbatim\n"
  "*Text inserted before a piece of literal text.
`texfax-prefix' should end in a newline.")

(defvar texfax-postfix "\\endverbatim\n"
  "*Text inserted after a piece of literal text.
`texfax-postfix' should end in a newline.")

(defvar texfax-table (list (cons "\\" "\\\\"))
  "*List of cons cells of the form

    (SEARCH-STRING . REPLACEMENT-TEXT)

which will be applied to each piece of literal text.")

(defvar texfax-tex-program nil
  "*TeX program name, defaults to \"tex\".")

(defvar texfax-tex-switches nil
  "*Extra arguments when `texfax-tex-program' is invoked.")

(defvar texfax-dvips-program nil
  "*DVIPS program name, defaults to \"dvips\".")

(defvar texfax-dvips-switches nil
  "*Extra arguments when `texfax-dvips-program' is invoked.")

(defun texfax-write-region (start end file-name)
  "Generate a TeX input file from the region between START and END,
process it with TeX and DVIPS and leave the PostScript output in FILE-NAME.
Intangible text will be treated as literal TeX input while the rest of the
text will be grouped between `texfax-prefix' and `texfax-postfix'."
  (interactive "r\nFFile name: ")
  (let ((tex-dir (or (file-name-directory file-name) "./"))
	(tex-name (concat file-name ".tex"))
	(dvi-name (concat file-name ".dvi"))
	(log-name (concat file-name ".log"))
	(orig-buf (current-buffer))
	(tex-buf nil))
    (unwind-protect
	(progn
	  (setq tex-buf (generate-new-buffer " texfax"))
	  (buffer-disable-undo tex-buf)
	  (set-buffer tex-buf)
	  (cd tex-dir)
	  (insert (save-excursion
		    (set-buffer orig-buf)
		    (buffer-substring start end)))
	  (let ((limit (make-marker)))
	    (goto-char (point-min))
	    (insert (or texfax-preamble ""))
	    (while (not (eobp))
	      (set-marker limit (or (text-property-any
				     (point) (point-max) 'intangible t)
				    (point-max)))
	      (and (< (point) limit)
		   (let ((table texfax-table)
			 begin search replace)
		     (untabify (point) limit)
		     (or (= (current-column) 0)
			 (insert ?\n))
		     (insert (or texfax-prefix ""))
		     (setq begin (point))
		     (while table
		       (goto-char begin)
		       (setq search (car (car table))
			     replace (cdr (car table)))
		       (while (search-forward search limit t)
			 (replace-match replace t t))
		       (setq table (cdr table)))
		     (goto-char limit)
		     (or (= (preceding-char) ?\n)
			 (insert ?\n))
		     (insert (or texfax-postfix ""))))
	      (goto-char (or (text-property-not-all
			      (point) (point-max) 'intangible t)
			     (point-max)))
	      (and (not (eobp))
		   (looking-at "\n")
		   (forward-char 1))
	      (insert "\\relax\n"))
	    (or (= (preceding-char) ?\n)
		(insert ?\n))
	    (insert (or texfax-postamble "")))
	  (write-region (point-min) (point-max) tex-name nil 0)
	  (or (zerop (apply 'call-process
			    (nconc (list (or texfax-tex-program "tex")
					 nil nil nil)
				   (append texfax-tex-switches
					   (list tex-name)))))
	      (error "TeX failed for some reason"))
	  (or (zerop (apply 'call-process
			    (nconc (list (or texfax-dvips-program "dvips")
					 nil nil nil)
				   (append texfax-dvips-switches
					   (list "-o" file-name dvi-name)))))
	      (error "DVIPS failed for some reason")))
      (if (file-exists-p tex-name)
	  (delete-file tex-name))
      (if (file-exists-p dvi-name)
	  (delete-file dvi-name))
      (if (file-exists-p log-name)
	  (delete-file log-name))
      (if tex-buf
	  (kill-buffer tex-buf))
      (set-buffer orig-buf))))


(provide 'texfax)


;;; texfax.el ends here
